'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function () {
  const callback = this.async();
  load(this).then(r => callback(null, r), e => callback(e, null));
};

var _asyncChildProcess = require('async-child-process');

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _loaderUtils = require('loader-utils');

var _loaderUtils2 = _interopRequireDefault(_loaderUtils);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _arraybufferToString = require('arraybuffer-to-string');

var _arraybufferToString2 = _interopRequireDefault(_arraybufferToString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const findSrcDir = (() => {
  var _ref = _asyncToGenerator(function* (childPath) {
    let candidate = childPath;

    while (candidate !== _path2.default.parse(candidate).root) {
      const maybeCargoFile = _path2.default.join(candidate, 'Cargo.toml');
      if (yield _fsExtra2.default.pathExists(maybeCargoFile)) {
        return candidate;
      }
      candidate = _path2.default.dirname(candidate);
    }

    return null;
  });

  return function findSrcDir(_x) {
    return _ref.apply(this, arguments);
  };
})();

const DEFAULT_OPTIONS = {
  release: false,
  gc: false,
  target: 'wasm32-unknown-unknown',
  cargoWeb: false,
  name: undefined,
  regExp: undefined
};

const loadCargoWeb = (() => {
  var _ref2 = _asyncToGenerator(function* (self, opts, srcDir) {
    const release = opts.release;
    const name = opts.name;

    const cmd = `cargo web build --message-format=json --target-webasm${release ? ' --release' : ''}`;
    const result = yield (0, _asyncChildProcess.execAsync)(cmd, { cwd: srcDir });

    const { wasmFile, jsFile } = handleCargo(self, result);

    if (!wasmFile) {
      throw new Error('No wasm file produced as build output');
    }
    if (!jsFile) {
      throw new Error('No js file produced as build output');
    }

    const jsData = yield _fsExtra2.default.readFile(jsFile);
    const wasmData = yield _fsExtra2.default.readFile(wasmFile);

    const context = opts.context || self.rootContext || self.options && self.options.context;
    const wasmOutFileName = _loaderUtils2.default.interpolateName(self, name, {
      context, content: wasmData, regExp: opts.regExp
    });

    self.emitFile(wasmOutFileName, wasmData);

    // Ugly way to do replaceAll... would be great to have some way to create a custom template here
    return (0, _arraybufferToString2.default)(jsData).split(`fetch( ${JSON.stringify(wasmFile)} )`).join(`fetch(__webpack_public_path__ + ${JSON.stringify(wasmOutFileName)})`).split(JSON.stringify(_path2.default.basename(wasmFile))).join(JSON.stringify(wasmOutFileName));
  });

  return function loadCargoWeb(_x2, _x3, _x4) {
    return _ref2.apply(this, arguments);
  };
})();

const loadRaw = (() => {
  var _ref3 = _asyncToGenerator(function* (self, opts, srcDir) {
    const release = opts.release;
    const gc = opts.gc;
    const target = opts.target;
    const cmd = `cargo build --message-format=json --target=${target}${release ? ' --release' : ''}`;

    const result = yield (0, _asyncChildProcess.execAsync)(cmd, { cwd: srcDir });

    let { wasmFile } = handleCargo(self, result);

    if (!wasmFile) {
      throw new Error('No wasm file produced as build output');
    }

    if (gc) {
      let gcWasmFile = wasmFile.substr(0, wasmFile.length - '.wasm'.length) + '.gc.wasm';
      yield (0, _asyncChildProcess.execAsync)(`wasm-gc ${wasmFile} ${gcWasmFile}`);
      wasmFile = gcWasmFile;
    }

    return yield _fsExtra2.default.readFile(wasmFile);
  });

  return function loadRaw(_x5, _x6, _x7) {
    return _ref3.apply(this, arguments);
  };
})();

const handleCargo = function (self, result) {
  let wasmFile;
  let jsFile;
  outer: for (let line of result.stdout.split(_os2.default.EOL)) {
    if (/^\s*$/.test(line)) {
      continue;
    }
    const data = JSON.parse(line);
    switch (data.reason) {
      case 'compiler-message':
        switch (data.message.level) {
          case 'warning':
            self.emitWarning(new Error(data.message.rendered));
            break;
          case 'error':
            self.emitError(new Error(data.message.rendered));
            break;
        }
        break;
      case 'compiler-artifact':
        if (!wasmFile) {
          wasmFile = data.filenames.find(p => p.endsWith('.wasm'));
        }
        if (!jsFile) {
          jsFile = data.filenames.find(p => p.endsWith('.js'));
        }
        if (wasmFile) {
          break outer;
        }
        break;
    }
  }
  return { wasmFile, jsFile };
};

const load = (() => {
  var _ref4 = _asyncToGenerator(function* (self) {
    const srcDir = yield findSrcDir(self.resourcePath);
    if (!srcDir) {
      throw new Error('No Cargo.toml file found in any parent directory.');
    }

    const opts = Object.assign({}, DEFAULT_OPTIONS, _loaderUtils2.default.getOptions(self));
    const cargoWeb = opts.cargoWeb;

    if (cargoWeb) {
      return yield loadCargoWeb(self, opts, srcDir);
    } else {
      return yield loadRaw(self, opts, srcDir);
    }
  });

  return function load(_x8) {
    return _ref4.apply(this, arguments);
  };
})();

;